@@ -71,6 +71,8 @@ json_print_attrs(struct lyout *out, int level, const struct lyd_node *node, cons
     size_t len;
     char *p;
 
+    LY_PRINT_SET;
+
     if (wdmod) {
         ly_print(out, "%*s\"%s:default\":\"true\"", LEVEL, INDENT, wdmod->name);
         ly_print(out, "%s%s", node->attr ? "," : "", (level ? "\n" : ""));
@@ -127,14 +129,14 @@ json_print_attrs(struct lyout *out, int level, const struct lyd_node *node, cons
 
         default:
             /* error */
-            ly_print(out, "\"(!error!)\"");
+            LOGINT(node->schema->module->ctx);
             return EXIT_FAILURE;
         }
 
         ly_print(out, "%s%s", attr->next ? "," : "", (level ? "\n" : ""));
     }
 
-    return EXIT_SUCCESS;
+    LY_PRINT_RET(node->schema->module->ctx);
 }
 
 static int
@@ -147,6 +149,8 @@ json_print_leaf(struct lyout *out, int level, const struct lyd_node *node, int o
     LY_DATA_TYPE datatype;
     size_t len;
 
+    LY_PRINT_SET;
+
     if ((node->dflt && (options & (LYP_WD_ALL_TAG | LYP_WD_IMPL_TAG))) ||
             (!node->dflt && (options & LYP_WD_ALL_TAG) && lyd_wd_default(leaf))) {
         /* we have implicit OR explicit default node */
@@ -212,7 +216,6 @@ contentprint:
             type = lyd_leaf_type((struct lyd_node_leaf_list *)leaf);
             if (!type) {
                 /* error */
-                ly_print(out, "\"(!error!)\"");
                 return EXIT_FAILURE;
             }
             datatype = type->base;
@@ -225,9 +228,17 @@ contentprint:
         ly_print(out, "[null]");
         break;
 
+    case LY_TYPE_UNKNOWN:
+        if (leaf->value_str[0] == '\0') {
+            ly_write(out, "\"\"", 2);
+            break;
+        }
+        datatype = ((struct lys_node_leaf *)leaf->schema)->type.base;
+        goto contentprint;
+
     default:
         /* error */
-        ly_print(out, "\"(!error!)\"");
+        LOGINT(node->schema->module->ctx);
         return EXIT_FAILURE;
     }
 
@@ -246,7 +257,7 @@ contentprint:
         ly_print(out, "%*s}", LEVEL, INDENT);
     }
 
-    return EXIT_SUCCESS;
+    LY_PRINT_RET(node->schema->module->ctx);
 }
 
 static int
@@ -254,6 +265,8 @@ json_print_container(struct lyout *out, int level, const struct lyd_node *node,
 {
     const char *schema;
 
+    LY_PRINT_SET;
+
     if (toplevel || !node->parent || nscmp(node, node->parent)) {
         /* print "namespace" */
         schema = lys_node_module(node->schema)->name;
@@ -270,9 +283,7 @@ json_print_container(struct lyout *out, int level, const struct lyd_node *node,
             return EXIT_FAILURE;
         }
         ly_print(out, "%*s}", LEVEL, INDENT);
-        if (node->child) {
-            ly_print(out, ",%s", (level ? "\n" : ""));
-        }
+        ly_print(out, "%s%s", (node->child? "," : ""), (level ? "\n" : ""));
     }
     if (json_print_nodes(out, level, node->child, 1, 0, options)) {
         return EXIT_FAILURE;
@@ -282,7 +293,7 @@ json_print_container(struct lyout *out, int level, const struct lyd_node *node,
     }
     ly_print(out, "%*s}", LEVEL, INDENT);
 
-    return EXIT_SUCCESS;
+    LY_PRINT_RET(node->schema->module->ctx);
 }
 
 static int
@@ -292,6 +303,8 @@ json_print_leaf_list(struct lyout *out, int level, const struct lyd_node *node,
     const struct lyd_node *list = node;
     int flag_empty = 0, flag_attrs = 0;
 
+    LY_PRINT_SET;
+
     if (is_list && !list->child) {
         /* empty, e.g. in case of filter */
         flag_empty = 1;
@@ -306,8 +319,8 @@ json_print_leaf_list(struct lyout *out, int level, const struct lyd_node *node,
     }
 
     if (flag_empty) {
-        ly_print(out, "%snull", (level ? " " : ""));
-        return EXIT_SUCCESS;
+        ly_print(out, "%s[]", (level ? " " : ""));
+        goto finish;
     }
     ly_print(out, "%s[%s", (level ? " " : ""), (level ? "\n" : ""));
 
@@ -407,7 +420,8 @@ json_print_leaf_list(struct lyout *out, int level, const struct lyd_node *node,
         ly_print(out, "%s%*s]", (level ? "\n" : ""), LEVEL, INDENT);
     }
 
-    return EXIT_SUCCESS;
+finish:
+    LY_PRINT_RET(node->schema->module->ctx);
 }
 
 static int
@@ -418,6 +432,8 @@ json_print_anydataxml(struct lyout *out, int level, const struct lyd_node *node,
     char *buf;
     const char *schema = NULL;
 
+    LY_PRINT_SET;
+
     if (toplevel || !node->parent || nscmp(node, node->parent)) {
         /* print "namespace" */
         schema = lys_node_module(node->schema)->name;
@@ -476,111 +492,140 @@ json_print_anydataxml(struct lyout *out, int level, const struct lyd_node *node,
     case LYD_ANYDATA_LYBD:
     case LYD_ANYDATA_LYB:
         /* other formats are not supported */
-        json_print_string(out, "(error)");
-        break;
+        LOGINT(node->schema->module->ctx);
+        return EXIT_FAILURE;
     }
 
     /* print attributes as sibling leaf */
-    if (node->attr) {
-        if (schema) {
-            ly_print(out, ",%s%*s\"@%s:%s\":%s{%s", (level ? "\n" : ""), LEVEL, INDENT, schema, node->schema->name,
-                     (level ? " " : ""), (level ? "\n" : ""));
-        } else {
-            ly_print(out, ",%s%*s\"@%s\":%s{%s", (level ? "\n" : ""), LEVEL, INDENT, node->schema->name,
-                     (level ? " " : ""), (level ? "\n" : ""));
+    switch(node->schema->nodetype) {
+    case LYS_ANYXML:
+        if (level) {
+            level--;
         }
-        if (json_print_attrs(out, (level ? level + 1 : level), node, NULL)) {
-            return EXIT_FAILURE;
+        if (is_object) {
+             ly_print(out, "%*s}", LEVEL, INDENT);
         }
-        ly_print(out, "%*s}", LEVEL, INDENT);
+        if (node->attr) {
+            if (schema) {
+                ly_print(out, ",%s%*s\"@%s:%s\":%s{%s", (level ? "\n" : ""), LEVEL, INDENT, schema, node->schema->name,
+                         (level ? " " : ""), (level ? "\n" : ""));
+            } else {
+                ly_print(out, ",%s%*s\"@%s\":%s{%s", (level ? "\n" : ""), LEVEL, INDENT, node->schema->name,
+                         (level ? " " : ""), (level ? "\n" : ""));
+            }
+            if (json_print_attrs(out, (level ? level + 1 : level), node, NULL)) {
+                return EXIT_FAILURE;
+            }
+            ly_print(out, "%*s}", LEVEL, INDENT);
+        }
+        break;
+    case LYS_ANYDATA:
+        if (node->attr) {
+            ly_print(out, ",%s%*s\"@\":%s{%s", (level ? "\n" : ""), LEVEL, INDENT, (level ? " " : ""), (level ? "\n" : ""));
+            if (json_print_attrs(out, (level ? level + 1 : level), node, NULL)) {
+                return EXIT_FAILURE;
+            }
+            ly_print(out, "%*s}", LEVEL, INDENT);
+        }
+        if (level) {
+            level--;
+        }
+        if (is_object) {
+            ly_print(out, "%s%*s}", (level ? "\n" : ""), LEVEL, INDENT);
+        }
+        break;
+    default:
+        LOGINT(node->schema->module->ctx);
+        return EXIT_FAILURE;
     }
 
-    if (level) {
-        level--;
-    }
-    if (is_object) {
-        ly_print(out, "%*s}", LEVEL, INDENT);
-    }
-
-    return EXIT_SUCCESS;
+    LY_PRINT_RET(node->schema->module->ctx);
 }
 
 static int
 json_print_nodes(struct lyout *out, int level, const struct lyd_node *root, int withsiblings, int toplevel, int options)
 {
-    int ret = EXIT_SUCCESS,comma_flag = 0;
+    int comma_flag = 0;
     const struct lyd_node *node, *iter;
 
-    LY_TREE_FOR(root, node) {
-        if (!lyd_wd_toprint(node, options)) {
-            /* wd says do not print */
-            continue;
-        }
+    LY_PRINT_SET;
 
-        switch (node->schema->nodetype) {
-        case LYS_RPC:
-        case LYS_ACTION:
-        case LYS_NOTIF:
-        case LYS_CONTAINER:
-            if (comma_flag) {
-                /* print the previous comma */
-                ly_print(out, ",%s", (level ? "\n" : ""));
-            }
-            ret = json_print_container(out, level, node, toplevel, options);
-            break;
-        case LYS_LEAF:
-            if (comma_flag) {
-                /* print the previous comma */
-                ly_print(out, ",%s", (level ? "\n" : ""));
-            }
-            ret = json_print_leaf(out, level, node, 0, toplevel, options);
-            break;
-        case LYS_LEAFLIST:
-        case LYS_LIST:
-            /* is it already printed? (root node is not) */
-            for (iter = node->prev; iter->next && node != root; iter = iter->prev) {
-                if (iter == node) {
-                    continue;
-                }
-                if (iter->schema == node->schema) {
-                    /* the list has alread some previous instance and therefore it is already printed */
-                    break;
-                }
-            }
-            if (!iter->next || node == root) {
+    LY_TREE_FOR(root, node) {
+        if (lyd_node_should_print(node, options)) {
+            /* wd says to print */
+            switch (node->schema->nodetype) {
+            case LYS_RPC:
+            case LYS_ACTION:
+            case LYS_NOTIF:
+            case LYS_CONTAINER:
                 if (comma_flag) {
                     /* print the previous comma */
                     ly_print(out, ",%s", (level ? "\n" : ""));
                 }
+                if (json_print_container(out, level, node, toplevel, options)) {
+                    return EXIT_FAILURE;
+                }
+                break;
+            case LYS_LEAF:
+                if (comma_flag) {
+                    /* print the previous comma */
+                    ly_print(out, ",%s", (level ? "\n" : ""));
+                }
+                if (json_print_leaf(out, level, node, 0, toplevel, options)) {
+                    return EXIT_FAILURE;
+                }
+                break;
+            case LYS_LEAFLIST:
+            case LYS_LIST:
+                /* is it already printed? (root node is not) */
+                for (iter = node->prev; iter->next && node != root; iter = iter->prev) {
+                    if (iter == node) {
+                        continue;
+                    }
+                    if (iter->schema == node->schema) {
+                        /* the list has alread some previous instance and therefore it is already printed */
+                        break;
+                    }
+                }
+                if (!iter->next || node == root) {
+                    if (comma_flag) {
+                        /* print the previous comma */
+                        ly_print(out, ",%s", (level ? "\n" : ""));
+                    }
 
-                /* print the list/leaflist */
-                ret = json_print_leaf_list(out, level, node, node->schema->nodetype == LYS_LIST ? 1 : 0, toplevel, options);
+                    /* print the list/leaflist */
+                    if (json_print_leaf_list(out, level, node, node->schema->nodetype == LYS_LIST ? 1 : 0, toplevel, options)) {
+                        return EXIT_FAILURE;
+                    }
+                }
+                break;
+            case LYS_ANYXML:
+            case LYS_ANYDATA:
+                if (comma_flag) {
+                    /* print the previous comma */
+                    ly_print(out, ",%s", (level ? "\n" : ""));
+                }
+                if (json_print_anydataxml(out, level, node, toplevel, options)) {
+                    return EXIT_FAILURE;
+                }
+                break;
+            default:
+                LOGINT(node->schema->module->ctx);
+                return EXIT_FAILURE;
             }
-            break;
-        case LYS_ANYXML:
-        case LYS_ANYDATA:
-            if (comma_flag) {
-                /* print the previous comma */
-                ly_print(out, ",%s", (level ? "\n" : ""));
-            }
-            ret = json_print_anydataxml(out, level, node, toplevel, options);
-            break;
-        default:
-            LOGINT(node->schema->module->ctx);
-            ret = EXIT_FAILURE;
-            break;
+
+            comma_flag = 1;
         }
 
         if (!withsiblings) {
             break;
         }
-        comma_flag = 1;
     }
     if (root && level) {
         ly_print(out, "\n");
     }
 
-    return ret;
+    LY_PRINT_RET(root ? root->schema->module->ctx : NULL);
 }
 
 int
@@ -589,6 +634,8 @@ json_print_data(struct lyout *out, const struct lyd_node *root, int options)
     const struct lyd_node *node, *next;
     int level = 0, action_input = 0;
 
+    LY_PRINT_SET;
+
     if (options & LYP_FORMAT) {
         ++level;
     }
@@ -642,5 +689,5 @@ json_print_data(struct lyout *out, const struct lyd_node *root, int options)
     ly_print(out, "}%s", (level ? "\n" : ""));
 
     ly_print_flush(out);
-    return EXIT_SUCCESS;
+    LY_PRINT_RET(NULL);
 }
