@@ -72,6 +72,20 @@ free_mlist(struct mlist **mlist) {
     }
 }
 
+inline static int
+is_netconf_filter(const struct lyd_node *node)
+{
+    return !strcmp(node->schema->name, "filter")
+            && (!strcmp(node->schema->module->name, "ietf-netconf")
+                    || !strcmp(node->schema->module->name, "notifications"));
+}
+
+inline static int
+is_type_or_select(const struct lyd_attr *attr)
+{
+    return (!strcmp(attr->name, "type") || !strcmp(attr->name, "select"));
+}
+
 static void
 xml_print_ns(struct lyout *out, const struct lyd_node *node, struct mlist **mlist, int options)
 {
@@ -86,9 +100,7 @@ xml_print_ns(struct lyout *out, const struct lyd_node *node, struct mlist **mlis
 
     /* add node attribute modules */
     for (attr = node->attr; attr; attr = attr->next) {
-        if (!strcmp(node->schema->name, "filter") &&
-                (!strcmp(node->schema->module->name, "ietf-netconf") ||
-                 !strcmp(node->schema->module->name, "notifications"))) {
+        if (is_netconf_filter(node) && is_type_or_select(attr)) {
             /* exception for NETCONF's filter attributes */
             continue;
         } else {
@@ -127,9 +139,7 @@ xml_print_ns(struct lyout *out, const struct lyd_node *node, struct mlist **mlis
         LY_TREE_FOR(node->child, node2) {
             LY_TREE_DFS_BEGIN(node2, next, cur) {
                 for (attr = cur->attr; attr; attr = attr->next) {
-                    if (!strcmp(cur->schema->name, "filter") &&
-                            (!strcmp(cur->schema->module->name, "ietf-netconf") ||
-                             !strcmp(cur->schema->module->name, "notifications"))) {
+                    if (is_netconf_filter(cur) && is_type_or_select(attr)) {
                         /* exception for NETCONF's filter attributes */
                         continue;
                     } else {
@@ -165,11 +175,11 @@ xml_print_attrs(struct lyout *out, const struct lyd_node *node, int options)
     struct lyd_attr *attr;
     const char **prefs, **nss;
     const char *xml_expr = NULL, *mod_name;
+    char *ident_val;
     uint32_t ns_count, i;
-    int rpc_filter = 0;
-    const struct lys_module *wdmod = NULL;
-    char *p;
+    int is_nc_filter = 0;
     size_t len;
+    const struct lys_module *wdmod = NULL;
 
     LY_PRINT_SET;
 
@@ -187,13 +197,44 @@ xml_print_attrs(struct lyout *out, const struct lyd_node *node, int options)
         }
     }
     /* technically, check for the extension get-filter-element-attributes from ietf-netconf */
-    if (!strcmp(node->schema->name, "filter")
-            && (!strcmp(node->schema->module->name, "ietf-netconf") || !strcmp(node->schema->module->name, "notifications"))) {
-        rpc_filter = 1;
+    if (is_netconf_filter(node)) {
+        is_nc_filter = 1;
     }
 
     for (attr = node->attr; attr; attr = attr->next) {
-        if (rpc_filter) {
+        ident_val = NULL;
+        if (attr->value_str && ((attr->value_type == LY_TYPE_INST) || (attr->value_type == LY_TYPE_IDENT)
+                || (!strcmp(attr->annotation->module->name, "yang") && !strcmp(attr->name, "key")))) {
+            if (attr->value_type == LY_TYPE_IDENT) {
+                ident_val = strchr(attr->value_str, ':');
+                assert(ident_val);
+                len = ident_val - attr->value_str;
+                mod_name = attr->annotation->module->name;
+                if (!strncmp(attr->value_str, mod_name, len) && !mod_name[len]) {
+                    /* skip local identity prefix */
+                    ++ident_val;
+                    goto normal_print;
+                }
+
+                /* print the prefix correctly below */
+                ident_val = NULL;
+            }
+
+            xml_expr = transform_json2xml(node->schema->module, attr->value_str, 1, &prefs, &nss, &ns_count);
+            if (!xml_expr) {
+                /* error */
+                return EXIT_FAILURE;
+            }
+
+            for (i = 0; i < ns_count; ++i) {
+                ly_print(out, " xmlns:%s=\"%s\"", prefs[i], nss[i]);
+            }
+            free(prefs);
+            free(nss);
+        }
+
+normal_print:
+        if (is_nc_filter && is_type_or_select(attr)) {
             /* exception for NETCONF's filter's attributes */
             if (!strcmp(attr->name, "select")) {
                 /* xpath content, we have to convert the JSON format into XML first */
@@ -229,46 +270,22 @@ xml_print_attrs(struct lyout *out, const struct lyd_node *node, int options)
         case LY_TYPE_UINT16:
         case LY_TYPE_UINT32:
         case LY_TYPE_UINT64:
+        case LY_TYPE_INST:
             if (attr->value_str) {
                 /* xml_expr can contain transformed xpath */
                 lyxml_dump_text(out, xml_expr ? xml_expr : attr->value_str, LYXML_DATA_ATTR);
             }
             break;
-
         case LY_TYPE_IDENT:
-            if (!attr->value_str) {
-                break;
-            }
-            p = strchr(attr->value_str, ':');
-            assert(p);
-            len = p - attr->value_str;
-            mod_name = attr->annotation->module->name;
-            if (!strncmp(attr->value_str, mod_name, len) && !mod_name[len]) {
-                lyxml_dump_text(out, ++p, LYXML_DATA_ATTR);
-            } else {
-                /* avoid code duplication - use instance-identifier printer which gets necessary namespaces to print */
-                goto printinst;
+            if (attr->value_str) {
+                if (ident_val) {
+                    lyxml_dump_text(out, ident_val, LYXML_DATA_ATTR);
+                } else {
+                    /* xml_expr can contain transformed xpath */
+                    lyxml_dump_text(out, xml_expr ? xml_expr : attr->value_str, LYXML_DATA_ATTR);
+                }
             }
             break;
-        case LY_TYPE_INST:
-printinst:
-            xml_expr = transform_json2xml(node->schema->module, ((struct lyd_node_leaf_list *)node)->value_str, 1,
-                                          &prefs, &nss, &ns_count);
-            if (!xml_expr) {
-                /* error */
-                return EXIT_FAILURE;
-            }
-
-            for (i = 0; i < ns_count; ++i) {
-                ly_print(out, " xmlns:%s=\"%s\"", prefs[i], nss[i]);
-            }
-            free(prefs);
-            free(nss);
-
-            lyxml_dump_text(out, xml_expr, LYXML_DATA_ATTR);
-            lydict_remove(node->schema->module->ctx, xml_expr);
-            break;
-
         /* LY_TYPE_LEAFREF not allowed */
         case LY_TYPE_EMPTY:
             break;
@@ -283,6 +300,7 @@ printinst:
 
         if (xml_expr) {
             lydict_remove(node->schema->module->ctx, xml_expr);
+            xml_expr = NULL;
         }
     }
 
@@ -642,7 +660,7 @@ xml_print_node(struct lyout *out, int level, const struct lyd_node *node, int to
 {
     int ret = EXIT_SUCCESS;
 
-    if (!lyd_toprint(node, options)) {
+    if (!lyd_node_should_print(node, options)) {
         /* wd says do not print */
         return EXIT_SUCCESS;
     }
