@@ -80,6 +80,18 @@
 #define LYXP_STRING_CAST_SIZE_START 64
 #define LYXP_STRING_CAST_SIZE_STEP 16
 
+/* Number of node checks when it starts being worth using the string dict.
+ * Node checks involves some string comparisons. Consider two ways to compare
+ * strings: one is using the string dict, another is simply comparing the chars
+ * of the strings (like 'strcmp' does). When using the string dict, the hash is
+ * computed using the entire string and after the memory address to that is got,
+ * the pointers are compared (as both strings compared are in the dict). It is
+ * worth using the dict when a lot of string comparisons using the same string
+ * is performed, but to compute just a few, comparing the chars of the strings
+ * is faster.
+ */
+#define LYXP_MIN_NODE_CHECKS_TO_USE_DICT 10
+
 /**
  * @brief Tokens that can be in an XPath expression.
  */
@@ -131,8 +143,8 @@ struct lyxp_expr {
     uint16_t *tok_len;       /* array of token lengths in expr */
     enum lyxp_expr_type **repeat; /* array of expression types that this token begins and is repeated ended with 0,
                                      more in the comment after this declaration */
-    uint16_t used;           /* used array items */
-    uint16_t size;           /* allocated array items */
+    uint32_t used;           /* used array items */
+    uint32_t size;           /* allocated array items */
 
     char *expr;              /* the original XPath expression */
 };
@@ -287,7 +299,7 @@ int lyxp_atomize(const char *expr, const struct lys_node *cur_snode, enum lyxp_n
 #define LYXP_SNODE_WHEN 0x10
 #define LYXP_SNODE_OUTPUT 0x20
 
-#define LYXP_SNODE_ALL 0x1C
+#define LYXP_SNODE_ALL 0x3C
 
 /**
  * @brief Works like lyxp_atomize(), but it is executed on all the when and must expressions
