@@ -1,7 +1,7 @@
 /**
- * @file test_sec7_8_3.c
+ * @file test_sec7_7_5.c
  * @author Pavol Vican
- * @brief Cmocka test for RFC 6020 section 7.8.3 conformance.
+ * @brief Cmocka test for RFC 6020 section 7.7.5 (also 7.7.1, 7.7) conformance.
  *
  * Copyright (c) 2016 CESNET, z.s.p.o.
  *
@@ -25,16 +25,18 @@
 #include "tests/config.h"
 #include "libyang.h"
 
-#define TEST_DIR "sec7_8_3"
-#define TEST_NAME test_sec7_8_3
-#define TEST_SCHEMA_COUNT 5
-#define TEST_SCHEMA_LOAD_FAIL 1,1,1,0,0
-#define TEST_DATA_FILE_COUNT 7
-#define TEST_DATA_FILE_LOAD_FAIL 1,1,0,1,1,0,0
+#define TEST_DIR "sec7_7_5"
+#define TEST_NAME test_sec7_7_5
+#define TEST_SCHEMA_COUNT 2
+#define TEST_SCHEMA_LOAD_FAIL 1,0
+#define TEST_DATA_FILE_COUNT 3
+#define TEST_DATA_FILE_LOAD_FAIL 1,0,0
+#define TEST_DATA_ORDERED_USER 0,0,0,1
 
 struct state {
     struct ly_ctx *ctx;
     struct lyd_node *node;
+    char *s;
 };
 
 static int
@@ -54,6 +56,7 @@ setup_f(void **state)
         fprintf(stderr, "Failed to create context.\n");
         return -1;
     }
+    st->s = NULL;
 
     return 0;
 }
@@ -65,6 +68,7 @@ teardown_f(void **state)
 
     lyd_free_withsiblings(st->node);
     ly_ctx_destroy(st->ctx, NULL);
+    free(st->s);
     free(st);
     (*state) = NULL;
 
@@ -72,7 +76,7 @@ teardown_f(void **state)
 }
 
 static void
-TEST_LIST_UNIQUE(void **state)
+TEST_LEAFLIST_ORDERED(void **state)
 {
     struct state *st = (*state);
     const int schemas_fail[] = {TEST_SCHEMA_LOAD_FAIL};
@@ -81,6 +85,8 @@ TEST_LIST_UNIQUE(void **state)
     LYS_INFORMAT schema_format = LYS_IN_YANG;
     const struct lys_module *mod;
     int i, j, ret;
+    const int data_ordered_user[] = {TEST_DATA_ORDERED_USER};
+    const char *user_data = "<l2>house</l2><l2>horse</l2><l2>car</l2>";
 
     for (i = 0; i < 2; ++i) {
         for (j = 0; j < TEST_SCHEMA_COUNT; ++j) {
@@ -100,6 +106,12 @@ TEST_LIST_UNIQUE(void **state)
                 assert_ptr_equal(st->node, NULL);
             } else {
                 assert_ptr_not_equal(st->node, NULL);
+                if (data_ordered_user[j]) {
+                    lyd_print_mem(&st->s, st->node, LYD_XML, LYP_WITHSIBLINGS | LYP_WD_ALL);
+                    assert_ptr_not_equal(strstr(st->s, user_data), NULL);
+                    free(st->s);
+                    st->s = NULL;
+                }
             }
             lyd_free_withsiblings(st->node);
             st->node = NULL;
@@ -144,7 +156,7 @@ int
 main(void)
 {
     const struct CMUnitTest tests[] = {
-        cmocka_unit_test_setup_teardown(TEST_LIST_UNIQUE, setup_f, teardown_f),
+        cmocka_unit_test_setup_teardown(TEST_LEAFLIST_ORDERED, setup_f, teardown_f),
     };
 
     return cmocka_run_group_tests(tests, NULL, NULL);
